# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.job_id import JobId
from .job_config import JobConfig

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Job(JobConfig):
    """
    A single unit of work that will execute asynchronously
    ---
    import datetime

    from flatfile import (
        Job,
        JobMode,
        JobStatus,
        JobSubject_Resource,
        JobType,
        Trigger,
    )

    Job(
        id="us_jb_YOUR_ID",
        created_at=datetime.datetime.fromisoformat(
            "2023-10-30 20:04:28.556000+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2023-10-30 20:04:32.075000+00:00",
        ),
        started_at=datetime.datetime.fromisoformat(
            "2023-10-30 20:04:29.453000+00:00",
        ),
        finished_at=datetime.datetime.fromisoformat(
            "2023-10-30 20:04:32.074000+00:00",
        ),
        environment_id="us_env_YOUR_ID",
        type=JobType.WORKBOOK,
        operation="submitAction",
        source="us_wb_YOUR_ID",
        destination="us_wb_YOUR_ID",
        trigger=Trigger.IMMEDIATE,
        status=JobStatus.COMPLETE,
        progress=100,
        file_id="us_fl_YOUR_ID",
        mode=JobMode.FOREGROUND,
        input={},
        subject=JobSubject_Resource(
            type="resource",
            id="us_wb_YOUR_ID",
        ),
        outcome={
            "message": "Data was successfully submitted to Webhook.site. Go check it out at https://example.site/example."
        },
        info="Starting job to submit action to webhook.site",
        managed=True,
    )
    """

    id: JobId
    created_at: dt.datetime = pydantic.Field(alias="createdAt", description="Date the item was created")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt", description="Date the item was last updated")
    started_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="startedAt", default=None, description="the time that the job started at"
    )
    finished_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="finishedAt", default=None, description="the time that the job finished at"
    )
    outcome_acknowledged_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="outcomeAcknowledgedAt",
        default=None,
        description="the time that the job's outcome has been acknowledged by a user",
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
