# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.commit_id import CommitId
from ..commons.types.errors import Errors
from ..commons.types.event_id import EventId
from ..commons.types.filter import Filter
from ..commons.types.filter_field import FilterField
from ..commons.types.record_id import RecordId
from ..commons.types.search_field import SearchField
from ..commons.types.search_value import SearchValue
from ..commons.types.sheet_id import SheetId
from ..commons.types.sort_direction import SortDirection
from ..commons.types.sort_field import SortField
from ..commons.types.success import Success
from ..commons.types.version_id import VersionId
from ..versions.types.version_response import VersionResponse
from .types.cell_value_union import CellValueUnion
from .types.get_records_response import GetRecordsResponse
from .types.record_data import RecordData
from .types.records import Records
from .types.records_response import RecordsResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RecordsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        sheet_id: SheetId,
        *,
        version_id: typing.Optional[VersionId] = None,
        commit_id: typing.Optional[CommitId] = None,
        since_version_id: typing.Optional[VersionId] = None,
        since_commit_id: typing.Optional[CommitId] = None,
        sort_field: typing.Optional[SortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filter: typing.Optional[Filter] = None,
        filter_field: typing.Optional[FilterField] = None,
        search_value: typing.Optional[SearchValue] = None,
        search_field: typing.Optional[SearchField] = None,
        ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]] = None,
        page_size: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
        include_counts: typing.Optional[bool] = None,
        include_length: typing.Optional[bool] = None,
        include_links: typing.Optional[bool] = None,
        include_messages: typing.Optional[bool] = None,
        fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        for_: typing.Optional[EventId] = None,
        q: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetRecordsResponse:
        """
        Returns records from a sheet in a workbook

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - version_id: typing.Optional[VersionId]. Deprecated, use `commitId` instead.

            - commit_id: typing.Optional[CommitId].

            - since_version_id: typing.Optional[VersionId]. Deprecated, use `sinceCommitId` instead.

            - since_commit_id: typing.Optional[CommitId].

            - sort_field: typing.Optional[SortField].

            - sort_direction: typing.Optional[SortDirection].

            - filter: typing.Optional[Filter].

            - filter_field: typing.Optional[FilterField]. Name of field by which to filter records

            - search_value: typing.Optional[SearchValue].

            - search_field: typing.Optional[SearchField].

            - ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]]. The Record Ids param (ids) is a list of record ids that can be passed to several record endpoints allowing the user to identify specific records to INCLUDE in the query, or specific records to EXCLUDE, depending on whether or not filters are being applied. When passing a query param that filters the record dataset, such as 'searchValue', or a 'filter' of 'valid' | 'error' | 'all', the 'ids' param will EXCLUDE those records from the filtered results. For basic queries that do not filter the dataset, passing record ids in the 'ids' param will limit the dataset to INCLUDE just those specific records. Maximum of 100 allowed.

            - page_size: typing.Optional[int]. Number of records to return in a page (default 10,000)

            - page_number: typing.Optional[int]. Based on pageSize, which page of records to return (Note - numbers start at 1)

            - include_counts: typing.Optional[bool]. **DEPRECATED** Use GET /sheets/:sheetId/counts

            - include_length: typing.Optional[bool]. The length of the record result set, returned as counts.total

            - include_links: typing.Optional[bool]. If true, linked records will be included in the results. Defaults to false.

            - include_messages: typing.Optional[bool]. Include error messages, defaults to false.

            - fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. A list of field keys to include in the response. If not provided, all fields will be included.

            - for_: typing.Optional[EventId]. if "for" is provided, the query parameters will be pulled from the event payload

            - q: typing.Optional[str]. An FFQL query used to filter the result set

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.records.get(
            sheet_id="us_sh_YOUR_ID",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "versionId": version_id,
                        "commitId": commit_id,
                        "sinceVersionId": since_version_id,
                        "sinceCommitId": since_commit_id,
                        "sortField": sort_field,
                        "sortDirection": sort_direction,
                        "filter": filter,
                        "filterField": filter_field,
                        "searchValue": search_value,
                        "searchField": search_field,
                        "ids": ids,
                        "pageSize": page_size,
                        "pageNumber": page_number,
                        "includeCounts": include_counts,
                        "includeLength": include_length,
                        "includeLinks": include_links,
                        "includeMessages": include_messages,
                        "fields": fields,
                        "for": for_,
                        "q": q,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetRecordsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, sheet_id: SheetId, *, request: Records, request_options: typing.Optional[RequestOptions] = None
    ) -> VersionResponse:
        """
        Updates existing records in a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - request: Records.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile import CellValue, Record, RecordConfig
        from flatfile.client import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.records.update(
            sheet_id="us_sh_YOUR_ID",
            request=[
                Record(
                    id="us_rc_YOUR_ID",
                    version_id="us_vr_YOUR_ID",
                    commit_id="us_vr_YOUR_ID",
                    values={
                        "firstName": CellValue(
                            messages=[],
                            valid=True,
                        ),
                        "lastName": CellValue(
                            messages=[],
                            valid=True,
                        ),
                        "email": CellValue(
                            messages=[],
                            valid=True,
                        ),
                    },
                    valid=True,
                    metadata={},
                    config=RecordConfig(),
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(VersionResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def insert(
        self,
        sheet_id: SheetId,
        *,
        request: typing.Sequence[RecordData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordsResponse:
        """
        Adds records to a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - request: typing.Sequence[RecordData].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile import CellValue
        from flatfile.client import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.records.insert(
            sheet_id="us_sh_YOUR_ID",
            request=[
                {
                    "firstName": CellValue(
                        messages=[],
                        valid=True,
                    ),
                    "lastName": CellValue(
                        messages=[],
                        valid=True,
                    ),
                    "email": CellValue(
                        messages=[],
                        valid=True,
                    ),
                }
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RecordsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        sheet_id: SheetId,
        *,
        ids: typing.Union[RecordId, typing.Sequence[RecordId]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Success:
        """
        Deletes records from a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - ids: typing.Union[RecordId, typing.Sequence[RecordId]]. A list of record IDs to delete. Maximum of 100 allowed.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.records.delete(
            sheet_id="us_sh_YOUR_ID",
            ids="us_rc_YOUR_ID",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "ids": ids,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Success, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def find_and_replace(
        self,
        sheet_id: SheetId,
        *,
        filter: typing.Optional[Filter] = None,
        filter_field: typing.Optional[FilterField] = None,
        search_value: typing.Optional[SearchValue] = None,
        search_field: typing.Optional[SearchField] = None,
        ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]] = None,
        q: typing.Optional[str] = None,
        find: typing.Optional[CellValueUnion] = OMIT,
        replace: typing.Optional[CellValueUnion] = OMIT,
        field_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VersionResponse:
        """
        Searches for all values that match the 'find' value (globally or for a specific field via 'fieldKey') and replaces them with the 'replace' value. Wrap 'find' value in double quotes for exact match (""). Returns a commitId for the updated records

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - filter: typing.Optional[Filter].

            - filter_field: typing.Optional[FilterField]. Name of field by which to filter records

            - search_value: typing.Optional[SearchValue].

            - search_field: typing.Optional[SearchField].

            - ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]]. The Record Ids param (ids) is a list of record ids that can be passed to several record endpoints allowing the user to identify specific records to INCLUDE in the query, or specific records to EXCLUDE, depending on whether or not filters are being applied. When passing a query param that filters the record dataset, such as 'searchValue', or a 'filter' of 'valid' | 'error' | 'all', the 'ids' param will EXCLUDE those records from the filtered results. For basic queries that do not filter the dataset, passing record ids in the 'ids' param will limit the dataset to INCLUDE just those specific records

            - q: typing.Optional[str]. An FFQL query used to filter the result set

            - find: typing.Optional[CellValueUnion]. A value to find for a given field in a sheet. For exact matches, wrap the value in double quotes ("Bob")

            - replace: typing.Optional[CellValueUnion]. The value to replace found values with

            - field_key: str. A unique key used to identify a field in a sheet

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import Flatfile

        client = Flatfile(
            token="YOUR_TOKEN",
        )
        client.records.find_and_replace(
            sheet_id="us_sh_YOUR_ID",
            field_key="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"fieldKey": field_key}
        if find is not OMIT:
            _request["find"] = find
        if replace is not OMIT:
            _request["replace"] = replace
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/find-replace"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "filter": filter,
                        "filterField": filter_field,
                        "searchValue": search_value,
                        "searchField": search_field,
                        "ids": ids,
                        "q": q,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(VersionResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRecordsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        sheet_id: SheetId,
        *,
        version_id: typing.Optional[VersionId] = None,
        commit_id: typing.Optional[CommitId] = None,
        since_version_id: typing.Optional[VersionId] = None,
        since_commit_id: typing.Optional[CommitId] = None,
        sort_field: typing.Optional[SortField] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        filter: typing.Optional[Filter] = None,
        filter_field: typing.Optional[FilterField] = None,
        search_value: typing.Optional[SearchValue] = None,
        search_field: typing.Optional[SearchField] = None,
        ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]] = None,
        page_size: typing.Optional[int] = None,
        page_number: typing.Optional[int] = None,
        include_counts: typing.Optional[bool] = None,
        include_length: typing.Optional[bool] = None,
        include_links: typing.Optional[bool] = None,
        include_messages: typing.Optional[bool] = None,
        fields: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        for_: typing.Optional[EventId] = None,
        q: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetRecordsResponse:
        """
        Returns records from a sheet in a workbook

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - version_id: typing.Optional[VersionId]. Deprecated, use `commitId` instead.

            - commit_id: typing.Optional[CommitId].

            - since_version_id: typing.Optional[VersionId]. Deprecated, use `sinceCommitId` instead.

            - since_commit_id: typing.Optional[CommitId].

            - sort_field: typing.Optional[SortField].

            - sort_direction: typing.Optional[SortDirection].

            - filter: typing.Optional[Filter].

            - filter_field: typing.Optional[FilterField]. Name of field by which to filter records

            - search_value: typing.Optional[SearchValue].

            - search_field: typing.Optional[SearchField].

            - ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]]. The Record Ids param (ids) is a list of record ids that can be passed to several record endpoints allowing the user to identify specific records to INCLUDE in the query, or specific records to EXCLUDE, depending on whether or not filters are being applied. When passing a query param that filters the record dataset, such as 'searchValue', or a 'filter' of 'valid' | 'error' | 'all', the 'ids' param will EXCLUDE those records from the filtered results. For basic queries that do not filter the dataset, passing record ids in the 'ids' param will limit the dataset to INCLUDE just those specific records. Maximum of 100 allowed.

            - page_size: typing.Optional[int]. Number of records to return in a page (default 10,000)

            - page_number: typing.Optional[int]. Based on pageSize, which page of records to return (Note - numbers start at 1)

            - include_counts: typing.Optional[bool]. **DEPRECATED** Use GET /sheets/:sheetId/counts

            - include_length: typing.Optional[bool]. The length of the record result set, returned as counts.total

            - include_links: typing.Optional[bool]. If true, linked records will be included in the results. Defaults to false.

            - include_messages: typing.Optional[bool]. Include error messages, defaults to false.

            - fields: typing.Optional[typing.Union[str, typing.Sequence[str]]]. A list of field keys to include in the response. If not provided, all fields will be included.

            - for_: typing.Optional[EventId]. if "for" is provided, the query parameters will be pulled from the event payload

            - q: typing.Optional[str]. An FFQL query used to filter the result set

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )
        await client.records.get(
            sheet_id="us_sh_YOUR_ID",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "versionId": version_id,
                        "commitId": commit_id,
                        "sinceVersionId": since_version_id,
                        "sinceCommitId": since_commit_id,
                        "sortField": sort_field,
                        "sortDirection": sort_direction,
                        "filter": filter,
                        "filterField": filter_field,
                        "searchValue": search_value,
                        "searchField": search_field,
                        "ids": ids,
                        "pageSize": page_size,
                        "pageNumber": page_number,
                        "includeCounts": include_counts,
                        "includeLength": include_length,
                        "includeLinks": include_links,
                        "includeMessages": include_messages,
                        "fields": fields,
                        "for": for_,
                        "q": q,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetRecordsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, sheet_id: SheetId, *, request: Records, request_options: typing.Optional[RequestOptions] = None
    ) -> VersionResponse:
        """
        Updates existing records in a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - request: Records.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile import CellValue, Record, RecordConfig
        from flatfile.client import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )
        await client.records.update(
            sheet_id="us_sh_YOUR_ID",
            request=[
                Record(
                    id="us_rc_YOUR_ID",
                    version_id="us_vr_YOUR_ID",
                    commit_id="us_vr_YOUR_ID",
                    values={
                        "firstName": CellValue(
                            messages=[],
                            valid=True,
                        ),
                        "lastName": CellValue(
                            messages=[],
                            valid=True,
                        ),
                        "email": CellValue(
                            messages=[],
                            valid=True,
                        ),
                    },
                    valid=True,
                    metadata={},
                    config=RecordConfig(),
                )
            ],
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(VersionResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def insert(
        self,
        sheet_id: SheetId,
        *,
        request: typing.Sequence[RecordData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RecordsResponse:
        """
        Adds records to a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - request: typing.Sequence[RecordData].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile import CellValue
        from flatfile.client import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )
        await client.records.insert(
            sheet_id="us_sh_YOUR_ID",
            request=[
                {
                    "firstName": CellValue(
                        messages=[],
                        valid=True,
                    ),
                    "lastName": CellValue(
                        messages=[],
                        valid=True,
                    ),
                    "email": CellValue(
                        messages=[],
                        valid=True,
                    ),
                }
            ],
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RecordsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        sheet_id: SheetId,
        *,
        ids: typing.Union[RecordId, typing.Sequence[RecordId]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Success:
        """
        Deletes records from a workbook sheet

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - ids: typing.Union[RecordId, typing.Sequence[RecordId]]. A list of record IDs to delete. Maximum of 100 allowed.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )
        await client.records.delete(
            sheet_id="us_sh_YOUR_ID",
            ids="us_rc_YOUR_ID",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/records"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "ids": ids,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Success, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Errors, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def find_and_replace(
        self,
        sheet_id: SheetId,
        *,
        filter: typing.Optional[Filter] = None,
        filter_field: typing.Optional[FilterField] = None,
        search_value: typing.Optional[SearchValue] = None,
        search_field: typing.Optional[SearchField] = None,
        ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]] = None,
        q: typing.Optional[str] = None,
        find: typing.Optional[CellValueUnion] = OMIT,
        replace: typing.Optional[CellValueUnion] = OMIT,
        field_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VersionResponse:
        """
        Searches for all values that match the 'find' value (globally or for a specific field via 'fieldKey') and replaces them with the 'replace' value. Wrap 'find' value in double quotes for exact match (""). Returns a commitId for the updated records

        Parameters:
            - sheet_id: SheetId. ID of sheet

            - filter: typing.Optional[Filter].

            - filter_field: typing.Optional[FilterField]. Name of field by which to filter records

            - search_value: typing.Optional[SearchValue].

            - search_field: typing.Optional[SearchField].

            - ids: typing.Optional[typing.Union[RecordId, typing.Sequence[RecordId]]]. The Record Ids param (ids) is a list of record ids that can be passed to several record endpoints allowing the user to identify specific records to INCLUDE in the query, or specific records to EXCLUDE, depending on whether or not filters are being applied. When passing a query param that filters the record dataset, such as 'searchValue', or a 'filter' of 'valid' | 'error' | 'all', the 'ids' param will EXCLUDE those records from the filtered results. For basic queries that do not filter the dataset, passing record ids in the 'ids' param will limit the dataset to INCLUDE just those specific records

            - q: typing.Optional[str]. An FFQL query used to filter the result set

            - find: typing.Optional[CellValueUnion]. A value to find for a given field in a sheet. For exact matches, wrap the value in double quotes ("Bob")

            - replace: typing.Optional[CellValueUnion]. The value to replace found values with

            - field_key: str. A unique key used to identify a field in a sheet

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from flatfile.client import AsyncFlatfile

        client = AsyncFlatfile(
            token="YOUR_TOKEN",
        )
        await client.records.find_and_replace(
            sheet_id="us_sh_YOUR_ID",
            field_key="email",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"fieldKey": field_key}
        if find is not OMIT:
            _request["find"] = find
        if replace is not OMIT:
            _request["replace"] = replace
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"sheets/{jsonable_encoder(sheet_id)}/find-replace"
            ),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "filter": filter,
                        "filterField": filter_field,
                        "searchValue": search_value,
                        "searchField": search_field,
                        "ids": ids,
                        "q": q,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(VersionResponse, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
