# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.space_id import SpaceId
from ...commons.types.user_id import UserId
from ...environments.types.guest_authentication_enum import GuestAuthenticationEnum
from .internal_space_config_base import InternalSpaceConfigBase
from .space_size import SpaceSize

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Space(InternalSpaceConfigBase):
    """
    A place to store your workbooks
    ---
    import datetime

    from flatfile import GuestAuthenticationEnum, Space

    Space(
        id="us_sp_YOUR_ID",
        name="My First Worbook",
        display_order=1,
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        last_activity_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        created_by_user_id="us_usr_YOUR_ID",
        workbooks_count=1,
        files_count=1,
        is_collaborative=True,
        upgraded_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        guest_authentication=[
            GuestAuthenticationEnum.MAGIC_LINK,
            GuestAuthenticationEnum.SHARED_LINK,
        ],
        environment_id="us_env_YOUR_ID",
        primary_workbook_id="us_wb_YOUR_ID",
        labels=[],
    )
    """

    id: SpaceId
    workbooks_count: typing.Optional[int] = pydantic.Field(
        alias="workbooksCount", default=None, description="Amount of workbooks in the space"
    )
    files_count: typing.Optional[int] = pydantic.Field(
        alias="filesCount", default=None, description="Amount of files in the space"
    )
    created_by_user_id: typing.Optional[UserId] = pydantic.Field(alias="createdByUserId", default=None)
    created_by_user_name: typing.Optional[str] = pydantic.Field(
        alias="createdByUserName", default=None, description="User name who created space"
    )
    created_at: dt.datetime = pydantic.Field(alias="createdAt", description="Date when space was created")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt", description="Date when space was updated")
    expired_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="expiredAt", default=None, description="Date when space was expired"
    )
    last_activity_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="lastActivityAt",
        default=None,
        description="This date marks the most recent activity within the space, tracking actions to the second. Activities include creating or updating records in a sheet, uploading files, or modifying a workbook's configuration.",
    )
    guest_link: typing.Optional[str] = pydantic.Field(
        alias="guestLink", default=None, description="Guest link to the space"
    )
    name: str = pydantic.Field(description="The name of the space")
    display_order: typing.Optional[int] = pydantic.Field(
        alias="displayOrder", default=None, description="The display order"
    )
    access_token: typing.Optional[str] = pydantic.Field(
        alias="accessToken", default=None, description="Access token for the space"
    )
    is_collaborative: typing.Optional[bool] = pydantic.Field(
        alias="isCollaborative", default=None, description="Flag for collaborative (project) spaces"
    )
    size: typing.Optional[SpaceSize] = pydantic.Field(default=None, description="Size information for the space")
    upgraded_at: typing.Optional[dt.datetime] = pydantic.Field(
        alias="upgradedAt", default=None, description="Date when the space was upgraded"
    )
    guest_authentication: typing.List[GuestAuthenticationEnum] = pydantic.Field(
        alias="guestAuthentication", description="Type of guest authentication"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
