# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.commit_id import CommitId
from ...commons.types.event_id import EventId
from ...commons.types.filter import Filter
from ...commons.types.filter_field import FilterField
from ...commons.types.record_id import RecordId
from ...commons.types.search_field import SearchField
from ...commons.types.search_value import SearchValue
from ...commons.types.sort_direction import SortDirection
from ...commons.types.sort_field import SortField
from ...commons.types.version_id import VersionId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ViewConfig(pydantic.BaseModel):
    """
    The configuration of a view. Filters, sorting, and search query.
    ---
    from flatfile import Filter, SortDirection, ViewConfig

    ViewConfig(
        filter=Filter.ERROR,
        filter_field="email",
        q="firstname like %John%",
        sort_field="email",
        sort_direction=SortDirection.ASC,
    )
    """

    version_id: typing.Optional[VersionId] = pydantic.Field(
        alias="versionId", default=None, description="Deprecated, use `commitId` instead."
    )
    commit_id: typing.Optional[CommitId] = pydantic.Field(alias="commitId", default=None)
    since_version_id: typing.Optional[VersionId] = pydantic.Field(
        alias="sinceVersionId", default=None, description="Deprecated, use `sinceCommitId` instead."
    )
    since_commit_id: typing.Optional[CommitId] = pydantic.Field(alias="sinceCommitId", default=None)
    sort_field: typing.Optional[SortField] = pydantic.Field(alias="sortField", default=None)
    sort_direction: typing.Optional[SortDirection] = pydantic.Field(alias="sortDirection", default=None)
    filter: typing.Optional[Filter] = None
    filter_field: typing.Optional[FilterField] = pydantic.Field(
        alias="filterField", default=None, description="Name of field by which to filter records"
    )
    search_value: typing.Optional[SearchValue] = pydantic.Field(alias="searchValue", default=None)
    search_field: typing.Optional[SearchField] = pydantic.Field(alias="searchField", default=None)
    ids: typing.Optional[typing.List[RecordId]] = pydantic.Field(
        default=None,
        description=(
            "The Record Ids param (ids) is a list of record ids that can be passed to several record endpoints allowing the user to identify specific records to INCLUDE in the query, or specific records to EXCLUDE, depending on whether or not filters are being applied. When passing a query param that filters the record dataset, such as 'searchValue', or a 'filter' of 'valid' \n"
            " 'error' \n"
            " 'all', the 'ids' param will EXCLUDE those records from the filtered results. For basic queries that do not filter the dataset, passing record ids in the 'ids' param will limit the dataset to INCLUDE just those specific records. Maximum of 100 allowed.\n"
        ),
    )
    page_size: typing.Optional[int] = pydantic.Field(
        alias="pageSize", default=None, description="Number of records to return in a page (default 10,000)"
    )
    page_number: typing.Optional[int] = pydantic.Field(
        alias="pageNumber",
        default=None,
        description="Based on pageSize, which page of records to return (Note - numbers start at 1)",
    )
    include_counts: typing.Optional[bool] = pydantic.Field(
        alias="includeCounts", default=None, description="**DEPRECATED** Use GET /sheets/:sheetId/counts"
    )
    include_length: typing.Optional[bool] = pydantic.Field(
        alias="includeLength", default=None, description="The length of the record result set, returned as counts.total"
    )
    include_links: typing.Optional[bool] = pydantic.Field(
        alias="includeLinks",
        default=None,
        description="If true, linked records will be included in the results. Defaults to false.",
    )
    include_messages: typing.Optional[bool] = pydantic.Field(
        alias="includeMessages", default=None, description="Include error messages, defaults to false."
    )
    for_: typing.Optional[EventId] = pydantic.Field(
        alias="for",
        default=None,
        description='if "for" is provided, the query parameters will be pulled from the event payload',
    )
    q: typing.Optional[str] = pydantic.Field(default=None, description="An FFQL query used to filter the result set")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
