# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.action import Action
from ...commons.types.environment_id import EnvironmentId
from ...commons.types.space_id import SpaceId
from ...sheets.types.sheet_config import SheetConfig
from .workbook_config_settings import WorkbookConfigSettings
from .workbook_treatments import WorkbookTreatments

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CreateWorkbookConfig(pydantic.BaseModel):
    """
    Properties used to create a new Workbook
    ---
    from flatfile import (
        Action,
        ActionMode,
        CreateWorkbookConfig,
        Property_String,
        SheetConfig,
        WorkbookConfigSettings,
    )

    CreateWorkbookConfig(
        name="My First Workbook",
        sheets=[
            SheetConfig(
                name="Contacts",
                slug="contacts",
                fields=[
                    Property_String(
                        type="string",
                        key="firstName",
                        label="First Name",
                    ),
                    Property_String(
                        type="string",
                        key="lastName",
                        label="Last Name",
                    ),
                    Property_String(
                        type="string",
                        key="email",
                        label="Email",
                    ),
                ],
                mapping_confidence_threshold=0.5,
            )
        ],
        labels=["simple-demo"],
        actions=[
            Action(
                operation="submitAction",
                mode=ActionMode.FOREGROUND,
                label="Submit",
                description="Submit data to webhook.site",
                primary=True,
            )
        ],
        settings=WorkbookConfigSettings(
            track_changes=True,
        ),
    )
    """

    name: str = pydantic.Field(description="The name of the Workbook.")
    labels: typing.Optional[typing.List[str]] = pydantic.Field(
        default=None, description="An optional list of labels for the Workbook."
    )
    space_id: typing.Optional[SpaceId] = pydantic.Field(
        alias="spaceId", default=None, description="Space to associate with the Workbook."
    )
    environment_id: typing.Optional[EnvironmentId] = pydantic.Field(
        alias="environmentId", default=None, description="Environment to associate with the Workbook"
    )
    namespace: typing.Optional[str] = pydantic.Field(
        default=None, description="Optional namespace to apply to the Workbook."
    )
    sheets: typing.Optional[typing.List[SheetConfig]] = pydantic.Field(
        default=None, description="Sheets to create on the Workbook."
    )
    actions: typing.Optional[typing.List[Action]] = pydantic.Field(
        default=None, description="Actions to create on the Workbook."
    )
    settings: typing.Optional[WorkbookConfigSettings] = pydantic.Field(
        default=None, description="The Workbook settings."
    )
    metadata: typing.Optional[typing.Any] = pydantic.Field(default=None, description="Metadata for the workbook")
    treatments: typing.Optional[typing.List[WorkbookTreatments]] = pydantic.Field(
        default=None, description="Treatments for the workbook"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}
